import pandas as pd
import numpy as np
import statistics

def column_stats(data, columns):
    """
  Obtain summary statistics of column(s) including count, mean, median, mode, Q1, Q3, 
  variance, standard deviation, correlation, and covariance in table format.

  Parameters
  -------------

  data: array_like
           The data set from which columns will be selected

  columns: vector of strings
            Columns for which to obtain summary stats, correlation matrix, and covariance matrix
            (if > 1 column arguments used)

  Returns
  -------------
  array
          Summary table detailing all statistics and correlations between chosen columns

  Examples
  -------------
  >>> column_stats(iris, ('sepal_width', 'petal_length'))
  >>>
  """
    if not isinstance(columns, (list, tuple, np.ndarray)):
        raise TypeError("'columns' should be a list, tuple, or array")
    for column in columns:
        if not isinstance(column, str):
            raise TypeError("item in columns should be of type string")
    if not isinstance(data, pd.DataFrame):
        raise TypeError("data should be a DataFrame")
        
    for column in columns:
        for row in data[column]:
            if isinstance(row, str):
                raise TypeError("values should be of type integer")
    
    
    statsdict = {'Column': [], 'Count': [], 'Mean': [], 'Median': [], 'Mode': [], 'Q1': [], 'Q3': [], 'Var': [], 'Stdev': []}
    for column in columns:
        statsdict['Column'].append(column)
        statsdict['Count'].append(round(float(data[column].describe().loc['count']), 3))
        statsdict['Mean'].append(round(float(data[column].describe().loc['mean']), 3))
        statsdict['Median'].append(round(float(data[column].describe().loc['50%']), 3))
        statsdict['Mode'].append(statistics.mode(data[column]))
        statsdict['Q1'].append(round(float(data[column].describe().loc['25%']), 3))
        statsdict['Q3'].append(round(float(data[column].describe().loc['75%']), 3))
        statsdict['Var'].append(round(data[column].var(), 3))
        statsdict['Stdev'].append(round(data[column].std(), 3))
        

    cols = []
    for column in columns:
        cols.append(column)
        
    covmatrix = pd.DataFrame(data, columns = cols)

    corrmatrix = pd.DataFrame(data, columns = cols)   
    return pd.DataFrame.from_dict(statsdict), corrmatrix.corr(), covmatrix.cov()



import pandas as pd
import altair as alt

def numeric_plots(df):
    """
    Creating a matrix of correlation plots with the numeric features.

    Parameters
    ----------
        df: pandas.dataframe
            A pandas dataframe

    Returns
    -------
        splom: Altair chart object
        The Altair object for the plots

    Example
    -------
    >>> from EDAhelper.numeric_plots import numeric_plots
    >>> splom = numeric_plot(df) 
    """

    # Data validation
    if not isinstance(df, pd.DataFrame):
        raise TypeError("'df' should be of type 'pandas.DataFrame'.")

    numeric_cols = df.select_dtypes(include=['float64', 'int64'])

    splom = alt.Chart(numeric_cols).mark_point(opacity=0.3, size=10).encode(
        alt.X(
            alt.repeat('row'),
            type='quantitative',
            scale=alt.Scale(zero=False)
        ),
        alt.Y(
            alt.repeat('column'),
            type='quantitative',
            scale=alt.Scale(zero=False)
        )
    ).properties(
        width=120,
        height=120
    ).repeat(
        column=list(numeric_cols.columns),
        row=list(numeric_cols.columns)
    ).configure_axis(
        labelFontSize=8,
        titleFontSize=8
    )

    return splom


print(numeric_plots.__doc__)
